function [alpha, tri] = sfg(x, y, p, q, tri, offset)
% SFG  Recover a surface from gradient values
%    [U] = SFG(X,Y,P,Q) returns the values of U at the points in the
%    vectors x, y such that U matches the component gradient values
%    given in the vectors p, q.  For uniqueness, U(X(1),Y(1)) will be
%    0.  A mesh will be generated inside the convex hull of x, y.
%    
%    [U,TRI] = SFG(X,Y,P,Q) additionally returns the triangulation
%    as a three-column matrix where each row represents the nodes
%    of a triangle.
%
%    [U,TRI] = SFG(X,Y,P,Q,TRI) specify a 3-column matrix where each
%    row represents the nodes of a triangle.  This mesh does NOT have
%    to be convex.
%
%    [U,TRI] = SFG(X,Y,P,Q,TRI,OFFSET) specify a value for
%    U(X(1),Y(1)).  Set TRI to [] if you want SFG to perform a
%    triangulation.
  
  plots = 1;  % make some plots
  
  if (nargin < 4)
    error('not enough input arguments');
  end
  if (nargin < 5) | (isempty(tri))
    tri = delaunay(x,y);
  end
  if (nargin < 6)
    offset = 0;
  end
  
  nnode = length(x);
  [ntri, temp] = size(tri);

  % build area and centroid data
  areas = zeros(ntri,1);
  centroids = zeros(ntri,2);
  for n = 1:ntri
    a = [x(tri(n,1)) y(tri(n,1))];
    b = [x(tri(n,2)) y(tri(n,2))];
    c = [x(tri(n,3)) y(tri(n,3))];
    areas(n) = triarea(a,b,c);
    centroids(n,:) = (a+b+c)/3;
  end

  if (plots)
    figure(10);  clf;  subplot(2,2,1);
    plot(x,y,'ro');  hold on;
    trimesh(tri, x, y, 'color', [0,0,0]);
    xlabel('x');  ylabel('y');
  end
  
  % allocate a sparse matrix
  A = spalloc(nnode,nnode,7*nnode);
  b = zeros(nnode, 1);

  % loop over all triangles
  for k = 1:ntri
    % indicies of nodes
    ni1 = tri(k,1);
    ni2 = tri(k,2);
    ni3 = tri(k,3);
    % location of nodes
    n1 = [x(ni1) y(ni1)];
    n2 = [x(ni2) y(ni2)];
    n3 = [x(ni3) y(ni3)];

    % coefficients of unit linear fcns on triangle
    [phi1x, phi1y, temp] = lineartri(n1,n2,n3, 1, 0, 0);
    [phi2x, phi2y, temp] = lineartri(n1,n2,n3, 0, 1, 0);
    [phi3x, phi3y, temp] = lineartri(n1,n2,n3, 0, 0, 1);
  
    A(ni1,ni1) = A(ni1,ni1) + areas(k)*(phi1x*phi1x + phi1y*phi1y);
    A(ni1,ni2) = A(ni1,ni2) + areas(k)*(phi1x*phi2x + phi1y*phi2y);
    A(ni2,ni1) = A(ni1,ni2);
    A(ni1,ni3) = A(ni1,ni3) + areas(k)*(phi1x*phi3x + phi1y*phi3y);
    A(ni3,ni1) = A(ni1,ni3);
    A(ni2,ni2) = A(ni2,ni2) + areas(k)*(phi2x*phi2x + phi2y*phi2y);
    A(ni2,ni3) = A(ni2,ni3) + areas(k)*(phi2x*phi3x + phi2y*phi3y);
    A(ni3,ni2) = A(ni2,ni3);
    A(ni3,ni3) = A(ni3,ni3) + areas(k)*(phi3x*phi3x + phi3y*phi3y);

    pav = (p(ni1) + p(ni2) + p(ni3))/3;
    qav = (q(ni1) + q(ni2) + q(ni3))/3;
    b(ni1) = b(ni1) + areas(k)*(pav*phi1x + qav*phi1y);
    b(ni2) = b(ni2) + areas(k)*(pav*phi2x + qav*phi2y);
    b(ni3) = b(ni3) + areas(k)*(pav*phi3x + qav*phi3y);
  end  

  if (plots)
    figure(10);  subplot(2,2,3);
    trimesh(tri, x, y, b);
    xlabel('x');  ylabel('y');  zlabel('b');
  end
  
  % crop the first node
  A = A(2:end,2:end);
  b = b(2:end);

  alpha= A\b;
  alpha = [0; alpha] + offset;

  if (plots)
    figure(10);  subplot(2,2,2);
    spy(A);
    title(['\kappa = ' num2str(condest(A))]);
    
    figure(10);  subplot(2,2,4);
    trimesh(tri, x, y, alpha);
    xlabel('x');  ylabel('y');  zlabel('u_h');
  end